<?php

declare(strict_types=1);

namespace Dedi\SyliusEcotaxPlugin\Calculator;

use Dedi\SyliusEcotaxPlugin\Entity\EcotaxInterface;
use Sylius\Component\Addressing\Matcher\ZoneMatcherInterface;
use Sylius\Component\Addressing\Model\ZoneInterface;
use Sylius\Component\Core\Exception\MissingChannelConfigurationException;
use Sylius\Component\Core\Model\AddressInterface;
use Sylius\Component\Core\Model\ChannelPricingInterface;
use Sylius\Component\Core\Model\ProductVariantInterface;
use Sylius\Component\Core\Model\Scope;
use Sylius\Component\Taxation\Calculator\CalculatorInterface;
use Sylius\Component\Taxation\Resolver\TaxRateResolverInterface;

final class EcotaxWithTaxCalculator implements EcotaxCalculatorInterface
{
    public function __construct(
        private EcotaxCalculatorInterface $ecotaxCalculator,
        private TaxRateResolverInterface $taxRateResolver,
        private CalculatorInterface $taxCalculator,
        private ZoneMatcherInterface $zoneMatcher,
    ) {
    }

    public function calculate(EcotaxInterface $ecotax, array $context): int
    {
        $price = $this->ecotaxCalculator->calculate($ecotax, $context);

        $zone = null;
        if (array_key_exists('address', $context) && $context['address'] instanceof AddressInterface) {
            $address = $context['address'];
            $zone = $this->getTaxZone($address);
        }

        /** @var ChannelPricingInterface|null $channelPricing */
        $channelPricing = $ecotax->getChannelPricing();
        if (null === $channelPricing || null === $channelPricing->getProductVariant()) {
            throw new MissingChannelConfigurationException(sprintf(
                'Ecotax %s has no channel pricing defined',
                $ecotax->getId(),
            ));
        }

        return $this->getPriceWithTax($channelPricing->getProductVariant(), $price, [
            'zone' => $zone,
        ]);
    }

    private function getPriceWithTax(ProductVariantInterface $variant, int $price, array $parameters = []): int
    {
        $taxRate = $this->taxRateResolver->resolve($variant, $parameters);

        if (null === $taxRate) {
            return $price;
        }

        $taxAmount = $this->taxCalculator->calculate($price, $taxRate);

        if ($taxRate->isIncludedInPrice()) {
            return $price;
        }

        return (int) ($price + $taxAmount);
    }

    private function getTaxZone(AddressInterface $address): ?ZoneInterface
    {
        return $this->zoneMatcher->match($address, Scope::TAX);
    }
}
